import json
import logging
import threading
import time

import amqp

logger = logging.getLogger(__name__)


class Server(object):
    def __init__(self, name='', prefix='ws', threaded=False, host='localhost', user='guest', password='guest',
                 vhost='/', dumper=None):
        if not prefix.endswith('_'):
            prefix += '_'
        if len(name) > 0 and not name.endswith('_'):
            name += '_'
        self.credentials = {
            'host': host,
            'userid': user,
            'password': password,
            'virtual_host': vhost
        }
        if dumper is None:
            dumper = json
        self.dumper = dumper
        self.prefix = prefix
        self.name = name
        self.exchange_name = self.prefix + 'exchange'
        self.queue_name = self.prefix + name + 'queue'
        self.threaded = threaded
        self.connection = None
        self.channel = None
        self.endpoints = {}

    def register_endpoint(self, name, endpoint):
        self.endpoints[(self.prefix + self.name + name).replace('_', '.')] = endpoint

    def connect(self):
        self.connection = amqp.Connection(**self.credentials)
        self.channel = self.connection.channel()
        self.channel.exchange_declare(self.exchange_name, 'topic')
        self.channel.basic_qos(0, 1, False)
        self.channel.queue_declare(self.queue_name)
        self.prepare_queues()

    def prepare_queues(self):
        for name in self.endpoints.keys():
            self.channel.queue_bind(self.queue_name, self.exchange_name, routing_key=name)
            self.channel.basic_consume(self.queue_name, callback=self.consume)
            logger.debug('Endpoint %s bound to %s', name, self.queue_name)

    def handle(self, message):
        logger.debug('Starting execution')
        endpoint = self.endpoints.get(message.delivery_info['routing_key'])
        try:
            msg = self.dumper.loads(message.body)
            result = endpoint(*msg.get('args', []), **msg.get('kwargs', {}))
        except Exception as e:
            result = e
        logger.debug('Execution ended')
        if message.properties.get('reply_to'):
            try:
                body = self.dumper.dumps(result)
            except Exception:
                body = self.dumper.dumps({'error': 'Error dump result'})
            logger.debug(
                'Sending a reply %s to %s with correlation id %s', body, message.properties['reply_to'],
                message.properties['correlation_id']
            )
            self.channel.basic_publish(
                amqp.Message(body, correlation_id=message.properties['correlation_id']),
                routing_key=message.properties['reply_to']
            )
        return result

    def consume(self, message):
        message.channel.basic_ack(message.delivery_tag)
        logger.debug('Message %s with routing key %s received', message.body, message.delivery_info['routing_key'])
        if self.threaded:
            p = threading.Thread(target=self.handle, args=(message,))
            p.daemon = True
            p.start()
        else:
            self.handle(message)

    def start(self):
        logger.debug('Start consuming')
        try:
            self.connect()
            while True:
                self.channel.wait()
        except KeyboardInterrupt:
            logger.debug('Stop consuming')
            self.channel.close()
            self.connection.close()
        except Exception as e:
            try:
                self.channel.close()
            except Exception:
                pass
            try:
                self.connection.close()
            except Exception:
                pass
            logger.error(e)
            logger.debug('Reconnect')
            time.sleep(5)
            self.start()



from distutils.core import setup
from setuptools import find_packages

setup(
    name='ws-amqp',
    version='0.2.8',
    packages=find_packages(exclude=['tests', 'example']),
    url='https://github.com/websuslik/ws-amqp',
    license='MIT',
    author='Yuri Voronkov',
    author_email='websuslik@gmail.com',
    description='',
    test_suite="nose.collector",
    install_requires=['amqp']
)



import json
import logging
import uuid

import amqp

logger = logging.getLogger(__name__)


class Client(object):
    def __init__(self, name='', prefix='ws', host='localhost', user='guest', password='guest', vhost='/', timeout=5,
                 dumper=None):
        if not prefix.endswith('_'):
            prefix += '_'
        if len(name) > 0 and not name.endswith('_'):
            name += '_'
        self.credentials = {
            'host': host,
            'userid': user,
            'password': password,
            'virtual_host': vhost
        }
        if dumper is None:
            dumper = json
        self.dumper = dumper
        self.timeout = timeout
        self.prefix = prefix
        self.name = name
        self.exchange_name = self.prefix + 'exchange'
        self.connection = None
        self.channel = None
        self.connect()

    def connect(self):
        self.connection = amqp.Connection(**self.credentials)
        self.channel = self.connection.channel()

    def close(self):
        self.channel.close()
        self.connection.close()

    def call(self, _name, *args, **kwargs):
        correlation_id = str(uuid.uuid4())
        reply = self.channel.queue_declare(exclusive=True).queue
        routing_key = (self.prefix + self.name + _name).replace('_', '.')
        body = self.dumper.dumps({'args': args, 'kwargs': kwargs})
        self.channel.basic_publish(
            amqp.Message(body, reply_to=reply, correlation_id=correlation_id),
            self.exchange_name, routing_key=routing_key,

        )
        logger.debug('Message %s with routing key %s published', body, routing_key)
        logger.debug('Waiting for reply in queue %s with correlation id %s', reply, correlation_id)

        message = {}

        def _reply(_message):
            logger.debug(
                'Reply %s with correlation id %s received', _message.body, _message.properties['correlation_id']
            )
            if _message.properties['correlation_id'] == correlation_id:
                message['result'] = self.dumper.loads(_message.body)
                self.channel.basic_ack(_message.delivery_tag)

        while True:
            self.channel.basic_consume(reply, callback=_reply)
            self.connection.drain_events(self.timeout)
            if 'result' in message:
                return message['result']

    def publish(self, _name, *args, **kwargs):
        routing_key = (self.prefix + self.name + _name).replace('_', '.')
        body = self.dumper.dumps({'args': args, 'kwargs': kwargs})
        self.channel.basic_publish(amqp.Message(body), self.exchange_name, routing_key=routing_key)
        logger.debug('Message %s with routing key %s published', body, routing_key)
